--!parent
--!encoding=utf-8

--****************************************************************************
-- @file LSW CORE LIB: SPRITE
-- @author Valentin Schmidt
-- @version 0.4
-- @desc
-- Simple sprite abstraction, factory method returns new sprites with a
-- classic EventEmitter interface (addListener, removeListener, ...)
--****************************************************************************

property _DispatcherClass
property _nextChannel

property _x, _y, _marg, _dw, _props
property _tmp

----------------------------------------
-- @constructor
----------------------------------------
on new (me)
	me._DispatcherClass = me.$.include(me.libdir&"dispatcher.ls")
	me._nextChannel = 1

	me._props = [:]
	me._props["alignment"] = "center"
	me._props["fontsize"] = 10
	me._props["fontStyle"] = "bold"
	me._props["font"] = "MS Sans Serif"

	me._tmp = new(#field)
	cnt = count(me._props)
	repeat with i = 1 to cnt
		do("me._tmp."&me._props.getPropAt(i)&"=me._props[i]")
	end repeat
	me._tmp.rect = rect(0,0,1024,0)

	-- button defaults
	me._x = 10
	me._y = 10
	me._marg = 10
	me._dw = 10

	return me
end

----------------------------------------
-- Factory method: creates and returns a new "scripted" sprite
-- @param {member} mem
-- @param {propList} [props]
-- @param {bool} [dispatchEvents=false]
-- @return {sprite}
----------------------------------------
on make (me, mem, props, dispatchEvents)
	s = channel(me._nextChannel).makeScriptedSprite(mem)
	if ilk(props)=#propList then
		cnt = count(props)
		repeat with i = 1 to cnt
			do("s."&props.getPropAt(i)&"=props[i]")
		end repeat
	end if
	if dispatchEvents then
		obj = me._DispatcherClass.new()
		obj.spriteNum = me._nextChannel
		s.scriptInstanceList.add(obj)
	end if
	me._nextChannel = me._nextChannel + 1
	return s
end

----------------------------------------
-- Deletes a "scripted" sprite
-- @param {integer|sprite} spriteNumOrRef
----------------------------------------
on delete (me, spriteNumOrRef)
	if ilk(spriteNumOrRef)=#sprite then spriteNumOrRef = spriteNumOrRef.spriteNum
	sprite(spriteNumOrRef).member = member(0)
	channel(spriteNumOrRef).removeScriptedSprite()
end

----------------------------------------
-- Creates member, sets some defaults that make sense
-- @param {symbol} type
-- @param {propList} [props]
----------------------------------------
on newMember (me, type, props)
	m = new(type)
	case (type) of
		#button:
			m.text = "OK"
			m.alignment = "center"
		#field:
			m.rect = rect(0,0,100,20)
			m.border = 1
			m.editable = 1
		#shape:
			m.rect = rect(0,0,100,100)
			m.shapeType = #rect
			m.filled = 1
			m.pattern = 1

	end case
	if ilk(props)=#propList then
		cnt = count(props)
		repeat with i = 1 to cnt
			do("m."&props.getPropAt(i)&"=props[i]")
		end repeat
	end if
	return m
end

----------------------------------------
-- creates a button sprite
----------------------------------------
on buttonCreate (me, txt, callback, centerFlag)
	me._tmp.text = txt
	w = me._tmp.charPosToLoc(txt.length+1)[1] + 2*me._marg --16

	if centerFlag then
		me._x = (_movie.stage.rect.width-w)/2
	else if me._x + w>_movie.stage.rect.width then
		me._x = 10
		me._y = me._y + 25
	end if

	m = new(#button)
	m.rect = rect(0,0,w,0)
	m.text = txt

	cnt = count(me._props)
	repeat with i = 1 to cnt
		do("m."&me._props.getPropAt(i)&"=me._props[i]")
	end repeat

	m.scriptText = "on mouseUp"&RETURN&callback&RETURN&"end"
	_movie.puppetSprite(me._nextChannel, TRUE)
	sprite(me._nextChannel).member = m
	sprite(me._nextChannel).loc = point(me._x, me._y)
	me._x = me._x + w + me._dw

	me._nextChannel = me._nextChannel + 1

	if centerFlag then
		me._y = me._y + 25
	end if
end
